################################################################################
#
# date:     2006/12/29
# modified: 2007/10/31
#
# corresponds to class rjBazi.BipropApp 
#
################################################################################

# modified rounding, which always (!) round .5 up
rndstd <- function(x) {
	
	y <- rep(NA, length(x))
	
		
	for (i in 1:length(x)) {
		if (x[i]-floor(x[i]) == 0.5) y[i] <- ceiling(x[i])
		else y[i] <- round(x[i])
	}
	
	if (is.matrix(x)) {
		y <- matrix(y, nrow = dim(x)[1])
		rownames(y) <- rownames(x)
		colnames(y) <- colnames(x)
	}
	else if (is.vector(x)) names(y) <- names(x)
	
	return(y)
}

# biproportional apportionment

bipropApp <-
function(districtMagnitudes, votes, appMethod = "DivStd", pq = NULL, nzz = FALSE,
	minConstr = NA, minConstrSuper = NA, algo = "ASmdpt|TTflpt", colM = NA,
	appMethodSuper = NULL, sorted = TRUE, resolveTies = FALSE, name = NULL) {
	
	# transformation
	if (length(algo) == 1 && algo == "all") 
		algo <- c("ASmdpt", "ASextr", "ASrand", "TTinte", "TTflpt", "ASmdpt|TTinte",
			"ASextr|TTinte", "ASrand|TTinte", "ASmdpt|TTflpt", "ASextr|TTflpt",
			"ASrand|TTflpt", "IPFP|TTinte", "IPFP|TTflpt")
	
	if (length(appMethod) > 1) {
		appMethod <- ifelse(is.element("DivStd", appMethod),  "DivStd", appMethod[1])
		warning(paste("Only one rounding method allowed. Using ", appMethod, 
			". Please rerun the problem with other rounding methods.", sep=""))
	}
	
	# some initial checks
	checkParams(list(districtMagnitudes = districtMagnitudes, votes = votes, 
		appMethod = appMethod, pq = pq, nzz = nzz, minConstr = minConstr, 
		minConstrSuper = minConstrSuper, algo = algo, colM = colM, 
		appMethodSuper = appMethodSuper))	
	
	# some inits
	rows <- dim(votes)[1]
	cols <- dim(votes)[2]
	givenColM <- !identical(colM, NA)
	presOfMinConstr <- !identical(minConstr, NA)
	presOfMinConstrSuper <- !identical(minConstrSuper, NA)	# 2007/10/31
	
	if (is.null(appMethodSuper)) appMethodSuper <- appMethod
	
	if (nzz) votesSuper <- colSums(rndstd(diag(1/districtMagnitudes)%*%votes))
	else votesSuper <- colSums(votes)
	
	if(sorted) {
		tmpOrder <- order(votesSuper, decreasing=T)
		votes <- votes[,tmpOrder]
		votesSuper <- votesSuper[tmpOrder]
		if (!identical(minConstr, NA)) minConstr <- minConstr[,tmpOrder]
		if (!identical(minConstrSuper, NA)) minConstrSuper <- minConstrSuper[tmpOrder] 
	}
		
	# row/col names
	if (is.null(rownames(votes))) 
		rownames(votes) <- mapply(paste, rep("D", rows) ,seq(1, rows), sep="")
	if (is.null(colnames(votes))) 
		colnames(votes) <- mapply(paste, rep("P", cols) ,seq(1, cols), sep="")	
	
	# transform data
	districtMagnitudes <- as.integer(districtMagnitudes)
	districtNames <- rownames(votes)
	partyNames <- colnames(votes)
	names(votesSuper) <- partyNames

	# init return object
	app <- list()
	app$name <- name
	appnames <- "Superapportionment"
	
	# superapportionment if not col marginal is given
	if (!givenColM) {
		# compute minimum constraints for superapportionment		
		if (presOfMinConstr) {
			if (presOfMinConstrSuper) newMinConstrSuper <- pmax(minConstrSuper, colSums(minConstr))
			else newMinConstrSuper <- colSums(minConstr)
		}	
		else if (presOfMinConstrSuper) newMinConstrSuper <- minConstrSuper
		else newMinConstrSuper <- NA
			
		# calculate superapportionment
		superAppRet <- monopropApp(housesize = sum(districtMagnitudes), 
						votes = votesSuper, appMethod = appMethodSuper, pq = pq,
						minConstr = newMinConstrSuper, resolveTies = T)
		superAppRet$input$appMethod <- appMethodSuper
		superAppRet$input$nzz <- nzz				
		noOfSuper <- dim(superAppRet[[2]]$multipleApp)[1]

	}
	else noOfSuper <- 1
	
	
	for (k in 1:noOfSuper) {
		if (!givenColM) {
			cSuper <- superAppRet[[2]]$multipleApp[k,]	# 2007/10/31
			cVotes <- votes[,cSuper > 0]
			cPartyNames <- partyNames[cSuper > 0]
			if (presOfMinConstr) currentMinConstr <- minConstr[, cSuper > 0]

			cColM <- cSuper[cSuper > 0]
		}
		else {
			cColM <- colM
			cVotes <- votes
			cPartyNames <- partyNames
		}
		
		# init data
		cColM <- as.integer(cColM)
		biApp <- .jnew("rjBazi/BipropApp", as.double(as.vector(t(cVotes))), districtMagnitudes, cColM, cPartyNames)
		if (presOfMinConstr) .jcall(biApp, returnSig = "V", "setMinConstr", as.integer(t(currentMinConstr)))	# 2007/10/31
		if (identical(pq, NA))
			.jcall(biApp, returnSig = "V", "calculateBipropApp", algo[1], appMethod)
		else	
			.jcall(biApp, returnSig = "V", "calculateBipropApp", algo[1], appMethod, pq)

		ret <- collectAppData(biApp, rows, cols, districtNames, cPartyNames, resolveTies)
		if (length(algo) > 1) { # add something here to omit calculation if errorCode != 0
			for (a in 2:length(algo)) {
				.jcall(biApp, returnSig ="V", "calculateBipropApp", algo[a])
				errorCodes <- .jcall(biApp, returnSig = "I", "getErrorCodes")
				tmpIter <- .jcall(biApp, "[I", "getAlgoPerformanceMeasures")				
				if (errorCodes != -1) tmpIter <- rep(NA, length(tmpIter))	#tmpIter[tmpIter==0] <- NA
				names(tmpIter) <- mapply(paste, rep(.jcall(biApp, "S", "getAlgoName"), length(tmpIter)), .jcall(biApp, "[S", "getAlgoPerformMeasureNames"), sep="")
				ret$Iterations <- c(ret$Iterations, tmpIter)
			}		
		}
		ret$votes <- cVotes
		ret$appMethod <- appMethod
		if (presOfMinConstr) ret$minConstr <- currentMinConstr	# 2007/10/31 minConstr[,cSuper>0]
		retLength <- length(ret)
		if (presOfMinConstr) ret <- ret[c(retLength-1, retLength, seq(1,retLength-2))]
		else ret <- ret[c(retLength, seq(1,retLength-1))]
		app[[k+1]] <- ret
		appnames <- c(appnames, paste("Subapportionment", k, sep=""))
	}
	app[[1]] <- superAppRet
	names(app) <- appnames
	
	# remove reference to Java-object
	rm(biApp)
	
	# some more information
	app$info <- list(district = TRUE, separate = FALSE, nzz = nzz)
	class(app) <- "RBaziApportionment"
	return(app)
}

collectAppData <- function(biApp, rows, cols, districtNames, partyNames, resolveTies) {
	
	app <- list()
	tmpApp <- list()
	errorCodes <- .jcall(biApp, returnSig = "I", "getErrorCodes")
	errorMsg <- .jcall(biApp, returnSig = "S", "getErrorMsg")
	
	#	browser()
	if (errorCodes == -1) {
		tmpApp <- list()
		tmpApp$apportionment <- matrix(.jcall(biApp, returnSig = "[I", "getSubApp"), rows, byrow = TRUE)
		rownames(tmpApp$apportionment) <- districtNames
		colnames(tmpApp$apportionment) <- partyNames
		cols <- dim(tmpApp$apportionment)[2]
		tmpApp$ties <- matrix(.jcall(biApp, returnSig = "[I", "getFlags"), rows, byrow = TRUE)
		rownames(tmpApp$ties) <- districtNames
		colnames(tmpApp$ties) <- partyNames
#		tmpApp$algo <- algo
		tmpApp$rowDiv <- .jcall(biApp, "[D", "getRowDiv")
		tmpApp$colDiv <- .jcall(biApp, "[D", "getColDiv")
		tmpApp$tiesInSubApp <- .jcall(biApp, "Z", "getTieInfo")
		tmpApp$Iterations <- .jcall(biApp, "[I", "getAlgoPerformanceMeasures")

		names(tmpApp$Iterations) <- mapply(paste, rep(.jcall(biApp, "S", "getAlgoName"), length(tmpApp$Iterations)), .jcall(biApp, "[S", "getAlgoPerformMeasureNames"), sep="")

		if (resolveTies && tmpApp$tiesInSubApp) {
			tmpRet <- .jcall(biApp, "[I", "getSubAppResolved")
			if (length(tmpRet) %% (rows*cols) == 0) {
				noOfSubapps <- length(tmpRet)/(rows*cols)
				resolvedApportionment <- list()
				length(resolvedApportionment) <- noOfSubapps
				for (m in 1:noOfSubapps) {
					currentApp <- matrix(tmpRet[((m-1)*rows*cols+1):(m*rows*cols)], rows, byrow = TRUE)
					rownames(currentApp) <- districtNames
					colnames(currentApp) <- partyNames
					resolvedApportionment[[m]] <- currentApp
				}
				tmpApp$resolvedApportionment <- resolvedApportionment
			}
			else stop("something went wrong in resolving ties")
		}
	}
	else {
		tmpApp$error <- errorCodes
		tmpApp$errorMsg <- errorMsg
	}	
	return(tmpApp)
}
