#setClass("baziDataSuper",
#	representation(votes = "numeric",
#					appMethod = "character",
#					pq = "numeric",
#					minRestr = "numeric",
#					maxRestr = "numeric",
#					directseats = "numeric",
#					resolveTies = "logical"),
#	prototype(votes = 0, appMethod = "DivStd", pq = NaN,
#				minRestr = NaN, maxRestr = NaN, directseats = NaN, resolveTies = FALSE))
#setClass("baziData",
#	representation(housesize = "numeric"),
#	prototype(housesize = 0, votes = 0, appMethod = "DivStd", pq = NaN,
#				minRestr = NaN, maxRestr = NaN, directseats = NaN, resolveTies = FALSE),
#	contains = "baziDataSuper")
#setClass("districtBaziData", 
#	representation(districtMagnitudes = "numeric",
#					nzz = "logical",
#					separate = "logical"),
#	prototype(districtMagnitudes = 0, votes = 0, appMethod = "DivStd", pq = 0,
#					minRestr = NaN, maxRestr = NaN, directSeats = NaN, nzz = FALSE,
#					separate = FALSE, resolveTies = FALSE),
#	contains = "baziDataSuper")
# ----------------- new -------------------

################################################################################
#
# date:     2007/07/17
# modified: 2007/07/17
#
# class definitions for the RBazi-package
# author: S. Maier
#
################################################################################


# baziData
setClass("baziData",
	representation(housesize = "numeric",
					votes = "numeric",
					appMethod = "character",
					pq = "numeric",
					minRestr = "numeric",
					maxRestr = "numeric",
					directSeats = "numeric",
					resolveTies = "logical"),
	prototype(housesize = 0, votes = 0, appMethod = "DivStd",
				pq = NaN, minRestr = NaN, maxRestr = NaN, 
				directSeats = NaN, resolveTies = FALSE),
	valid = function(object) {
				
				# some inits
				allowedAppMethods <- c("DivStat", "DivDwn", "DivAbr", "DivStd", "DivUp", "DivAuf", "DivGeo", "DivHar", "DivPMean")
				if (!is.element(object@appMethod, allowedAppMethods)) stop("Please check apportionment method.")
				if (object@appMethod == "DivStat" && (is.na(object@pq) || object@pq <= 0 || object@pq >= 1)) stop("Parameter for stationary rounding has to be strictly between 0 and 1.")
				
				if (is.na(object@votes)) stop("Nothing to do.")
				if (object@housesize < 0 || any(object@votes < 0)) stop("Negative input.")
				
				noOfParties <- length(object@votes)
#				parties <- names(object@votes)
				
				if(!is.na(object@minRestr)) {
					if (length(object@minRestr) != noOfParties) stop("Length of votes and minimum restrictions differ.")
#					if (!identical(names(object@minRestr), parties) stop("Different ordering of votes and minimum restrictions.")
				}
				if(!is.na(object@maxRestr)) {
					if (length(object@maxRestr) != noOfParties) stop("Length of votes and minimum restrictions differ.")
#					if (!identical(names(object@maxRestr), parties) stop("Different ordering of votes and minimum restrictions.")
				}
				if(!is.na(object@directSeats)) {
					if (length(object@directSeats) != noOfParties) stop("Length of votes and minimum restrictions differ.")
#					if (!identical(names(object@directSeats), parties) stop("Different ordering of votes and minimum restrictions.")
				}
				
			})
				
# districtBaziData
setClass("districtBaziData",
	representation(districtMagnitudes = "numeric",
					votes = "matrix",
					appMethod = "character",
					pq = "numeric",
					minRestr = "matrix",
					maxRestr = "matrix",
					directSeats = "matrix",
					nzz = "logical",
					separate = "logical",
					resolveTies = "logical"),
	prototype(districtMagnitudes = c(0,0),
				votes = matrix(rep(0, 2), nrow = 2), appMethod = "DivStd",
				pq = NaN, minRestr = matrix(), maxRestr = matrix(), 
				directSeats = matrix(), 
				nzz = FALSE, separate = FALSE,
				resolveTies = FALSE),
	valid = function(object) {
				
				# some inits
				valid <- TRUE
				allowedAppMethods <- c("DivStat", "DivDwn", "DivAbr", "DivStd", "DivUp", "DivAuf", "DivGeo", "DivHar", "DivPMean")
				if (!is.element(object@appMethod, allowedAppMethods)) {
					warning("Please check apportionment method.")
					valid <- FALSE
				}	
				if (object@appMethod == "DivStat" && (is.na(object@pq) || object@pq <= 0 || object@pq >= 1)) {
					warning("Parameter for stationary rounding has to be strictly between 0 and 1.")
					valid <- FALSE
				}
				if (is.null(object@votes)) {
					warning("No votes given.")
					valid <- FALSE
				}	
				if (any(object@districtMagnitudes < 0) || any(object@votes < 0)) {
					warning("Negative input.")
					valid <- FALSE
				}	
				
				dimV <- dim(object@votes)
				
				if (object@separate && object@nzz) {
					warning("Clash of district options.")
					valid <- FALSE
				}	
				if (!object@separate && !is.na(object@directSeats)) warning("Directseats are not used in biproportional methods.")
				
				if(!any(is.na(object@minRestr))) {
					if (!identical(dim(object@minRestr), dimV)) stop("Dimension of votes and minimum restrictions differ.")
#					if (!identical(names(object@minRestr), parties) stop("Different ordering of votes and minimum restrictions.")
					valid <- FALSE
				}
				if(!any(is.na(object@maxRestr))) {
					if (identical(dim(object@maxRestr), dimV)) stop("Dimension of votes and minimum restrictions differ.")
#					if (!identical(names(object@maxRestr), parties) stop("Different ordering of votes and minimum restrictions.")
					valid <- FALSE
				}
				if(!any(is.na(object@directSeats))) {
					if (!identical(dim(object@directSeats), dimV)) stop("Dimension of votes and minimum restrictions differ.")
#					if (!identical(names(object@directSeats), parties) stop("Different ordering of votes and minimum restrictions.")
					valid <- FALSE
				}
				
				return(valid)
				
			})
			
setClassUnion("superBaziData", c("baziData", "districtBaziData"))			