################################################################################
#
# date:     2007/10/06
# modified: 2007/10/11
#
# some methods for RBazi-package
#
################################################################################


print.RBaziData = function(x, ...) {

	cat(x$name, "\n\n")
	
	distrData <- ifelse (is.element("districtMagnitudes", names(x)), TRUE, FALSE)
	
	appMethods <- x$appMethod
	if (is.element("DivPot", appMethods)) {
		ind <- which(appMethods == "DivPot")
		appMethods[ind] <- paste("DivPot (p=", x$pq, ")", sep="")
		appMethods <- paste(appMethods, sep="", collapse = ", ")
	}
	if (is.element("DivSta", appMethods)) {
		ind <- which(appMethods == "DivSta")
		appMethods[ind] <- paste("DivSta (q=", x$pq, ")", sep="")
		appMethods <- paste(appMethods, sep="", collapse = ", ")
	}
	
	if (is.element("error", names(x))) {
		print(x$errorMsg)
	
	}
	else if (distrData) {
		cat("Apportionment in", length(x$districtMagnitudes), "districts using",
			appMethods, "with district option",
			 ifelse(x$nzz, "NZZ", ifelse(x$separate, "separate", "biprop")), 
			 "\nDistricts and district magnitudes\n")
		print(x$districtMagnitudes)	
		cat("\n")
		if (is.element("minConstrSuper", names(x)) && !identical(x$minConstrSuper, NA)) {
			cat("Minimum requirements for superapportionment\n")
			print(x$minConstrSuper)
			cat("\n")
		}
		inputNames <- c()
		lC <- FALSE
		dS <- FALSE
		mR <- FALSE
		MR <- FALSE
		if (x$separate) {
			if (!(identical(x$listCombination, NA) || identical(x$listCombination, NULL))) {
				inputNames <- c(inputNames, "List Combination")
				lC <- TRUE
			cat("List combination 0 means \"not member of a list combination\"\n")				
			}	
			if (!(identical(x$directSeats, NA) || identical(x$directSeats, NULL))) {
				inputNames <- c(inputNames, "Direct Seats")
				dS <- TRUE
			}	
		}
		if (!identical(x$minConstr, NA)) {
			inputNames <- c(inputNames, "Minimum Requirements")
			mR <- TRUE
		}	
		if (x$separate && !(identical(x$maxConstr, NA) || identical(x$maxConstr, NULL))) {
			inputNames <- c(inputNames, "Maximum Requirements")
			MR <- TRUE
		}
		noOfDistr <- length(x$districtMagnitudes)
		noOfParties <- dim(x$votes)[2]
		noOfInputs <- length(inputNames) + 1
		
		partyNames <- colnames(x$votes)
		
		tmpOut <- matrix(nrow = noOfDistr, ncol = noOfParties*noOfInputs)
		tmpOutNames <- c()
		
		if (lC || dS || mR || MR) {
			for (j in 1:noOfParties) {
				tmpOut[,(j-1)*noOfInputs+1] <- x$votes[,j]
				jj <- 2
				if (lC) {
					tmpOut[,(j-1)*noOfInputs+jj] <- x$listCombination[,j]
					jj <- jj + 1
				}
				if (dS) {
					tmpOut[,(j-1)*noOfInputs+jj] <- x$directSeats[,j]
					jj <- jj + 1
				}
				if (mR) {
					tmpOut[,(j-1)*noOfInputs+jj] <- x$minConstr[,j]
					jj <- jj + 1
				}			
				if (MR) {
					tmpOut[,(j-1)*noOfInputs+jj] <- x$maxConstr[,j]
					jj <- jj + 1
				}
				tmpOutNames <- c(tmpOutNames, partyNames[j], inputNames)
			}

			rownames(tmpOut) <- rownames(x$votes)
			colnames(tmpOut) <- tmpOutNames
		}
		else {
			tmpOut <- x$votes
		}	
	}
	else {
		cat("Apportionment using", appMethods, 
			"for housesize(s)", paste(x$housesize, sep="", collapse=", "), "\n")
		tmpOut <- x$votes
		tmpOutNames <- "Votes"
		if (!identical(x$listCombination, NA)) {
			tmpOut <- cbind(tmpOut, x$listCombination)
			tmpOutNames <- c(tmpOutNames, "List Combination")
			cat("List combination 0 means \"not member of a list combination\"\n")
		}
		if (!identical(x$minConstr, NA)) {
			tmpOut <- cbind(tmpOut, x$minConstr)
			tmpOutNames <- c(tmpOutNames, "Minimum Requirements")
		}
		if (!identical(x$maxConstr, NA)) {
			tmpOut <- cbind(tmpOut, x$maxConstr)
			tmpOutNames <- c(tmpOutNames, "Maximum Requirements")
		}
		if (!identical(x$directSeats, NA)) {
			tmpOut <- cbind(tmpOut, x$directSeats)
			tmpOutNames <- c(tmpOutNames, "Direct Seats")
		}
		if (!identical(x$naiv, NA)) {
			tmpOut <- cbind(tmpOut, x$naiv)
			tmpOutNames <- c(tmpOutNames, "Sub")
		}	
		
		tmpOut <- as.matrix(tmpOut)
		
		colnames(tmpOut) <- tmpOutNames
	}

	print(tmpOut)
}

print.RBaziApportionment = function(x, ...) {

	xNames <- names(x)
	
	if (is.element("Superapportionment", xNames)) {	# biprop
		
		noOfSubapps <- length(xNames) - 1
		districtNames <- rownames(x[[2]]$votes)
		
		cat("Matrix apportionment\nSuperapportionment\n")
		print(x$Superapportionment)
		
		for (a in 2:noOfSubapps) {
			mC <- is.element("minConstr", names(x[[a]]))
			if (mC) tmpMin <- colSums(x[[a]]$minConstr)
			tmpApp <- colSums(x[[a]]$apportionment)
			rows <- dim(x[[a]]$votes)[1]
			cols <- dim(x[[a]]$votes)[2]
			partyNames <- colnames(x[[a]]$votes)
			outputs <- ifelse(mC, 3, 2)
			tmpOut <- matrix(nrow = rows, ncol = cols*outputs)
			tmpDiv <- c()
			tmpOutNames <- c()
			for (j in 1:cols) {
			
				tmpOut[,((j-1)*outputs+1)] <- x[[a]]$votes[,j]
				tmpDiv <- c(tmpDiv, x[[a]]$colDiv[j])
				tmpOutNames <- c(tmpOutNames, partyNames[j])
				if (mC) {
					tmpOut[,((j-1)*outputs+2)] <- x[[a]]$minConstr[,j]
					tmpOutNames <- c(tmpOutNames, "Min")
					tmpDiv <- c(tmpDiv, tmpMin[j])
				}	
				tmpOut[,(j*outputs)] <- x[[a]]$apportionment[,j]
				tmpOutNames <- c(tmpOutNames, x[[a]]$appMethod)
				tmpDiv <- c(tmpDiv, tmpApp[j])
			}	
			
			tmpOut <- cbind(tmpOut, rowSums(x[[a]]$apportionment), x[[a]]$rowDiv)
			tmpDiv <- c(tmpDiv, sum(tmpApp), NA)
			tmpOutNames <- c(tmpOutNames, "District Magnitude", "Divisor")
			tmpOut <- rbind(tmpOut, tmpDiv)
			rownames(tmpOut) <- c(districtNames, "Divisor")
			colnames(tmpOut) <- tmpOutNames
			cat(paste("\nSubapportionment", ifelse(noOfSubapps==2, "", 
				paste(" #", a-1, sep="")), "\n", sep=""))
			print(tmpOut)
			#print(unclass(x[[a]]))
		}
		
		#print(unclass(x))
	}
	else if (is.element("apportionment", xNames)) {	# district
		#message("Not yet completely implemented")
		cat("Districtwise apportionment\n")
		print(unclass(x))
		dA <- is.element("districtApportionments", names(x))
		if (dA) {
			for (i in 1:length(x$districtApportionments)) print(x$districtApportionments[[i]])
		}	

	}
	else {	# vector
		tmpOut <- x$input$votes
		tmpOutNames <- c("Votes")
		inputNames <- names(x$input)
		mC <- is.element("minConstr", inputNames)
		MC <- is.element("maxConstr", inputNames)
		oH <- is.element("directSeats", inputNames)
		
		if (mC) {
			tmpOut <- cbind(tmpOut, x$input$minConstr)
			tmpOutNames <- c(tmpOutNames, "Min")
		}
		if (MC) {
			tmpOut <- cbind(tmpOut, x$input$maxConstr)
			tmpOutNames <- c(tmpOutNames, "Max")
		}
		if (!is.null(x$name)) cat("Vector apportionment of ", x$name, "\n")
		for (a in 2:length(x)) {
			
			if (is.element("Summary", names(x[[a]]))) lC <- TRUE
			else lC <- FALSE

			if (lC)	{
				tmpOut <- 
					cbind(tmpOut, x[[a]]$Summary$apportionment[order(
						names(x[[a]]$Summary$apportionment))])
			} else tmpOut <- cbind(tmpOut, x[[a]]$apportionment)

			tmpOutNames <- c(tmpOutNames, names(x)[a])
			if (oH) {
				if (lC) {
					tmpOut <- cbind(tmpOut, x[[a]]$Summary$overhang[order(
						names(x[[a]]$Summary$overhang))])
				}else tmpOut <- cbind(tmpOut, x[[a]]$overhang)
				tmpOutNames <- c(tmpOutNames, "Overhang")
			}	
			
		}
		colnames(tmpOut) <- tmpOutNames
		print(tmpOut)
	
	}
}