################################################################################
#
# date:     2006/12/23
# modified: 2006/10/11
#
# corresponds to class rjBazi.MonopropApp 
#
################################################################################



# monoproportional apportionment
monopropApp <- 
function(housesize, votes, appMethod = "DivStd", pq = NULL, minConstr = NA, maxConstr=NA, directSeats = NA, sorted = F, resolveTies = F, listCombination = NA, name = NULL)
{

	# some initial checks
	checkParams(list(housesize = housesize, votes = votes, appMethod = appMethod,
		pq = pq, minConstr = minConstr, maxConstr = maxConstr, directSeats = directSeats))
	
	presOfMinConstr <- ifelse(identical(minConstr, NA) || identical(minConstr, NULL), FALSE, TRUE)
	presOfMaxConstr <- ifelse(identical(maxConstr, NA) || identical(maxConstr, NULL), FALSE, TRUE)
	presOfOverhang <- ifelse(identical(directSeats, NA) || identical(directSeats, NULL), FALSE, TRUE)
	
	if (sorted) {
		newOrder <- order(votes, decreasing = T)
		if (presOfMinConstr) minConstr <- minConstr[newOrder]
		if (presOfMaxConstr) maxConstr <- maxConstr[newOrder]
		if (presOfOverhang) directSeats <- directSeats[newOrder]
		if (!identical(listCombination, NA)) listCombination <- listCombination[newOrder]
		votes <- votes[newOrder]	# sort(votes, decreasing = T)
	}	
	
	# init party names if there are none
	if (is.null(names(votes))) names(votes) <- mapply(paste, rep("P", length(votes)) ,seq(1,length(votes)), sep="")
	partyNames <- names(votes)
	
	# init rjBazi
	if (identical(listCombination, NA)) {
		monoApp <- .jnew("rjBazi/MonopropApp", votes, partyNames)
			
		if (presOfMinConstr) 
			.jcall(monoApp, returnSig = "V", "setMinConstr", as.integer(minConstr))
			
		if (presOfMaxConstr)
			.jcall(monoApp, returnSig ="V", "setMaxConstr", as.integer(maxConstr))
	}		
	
	# init return object
	app <- list()
	namesApp <- c("input")
	app[[1]] <- list()
	app[[1]]$name <- name
	app[[1]]$votes <- votes
	if (presOfMinConstr) app[[1]]$minConstr <- minConstr
	if (presOfMaxConstr) app[[1]]$maxConstr <- maxConstr
	if (presOfOverhang) app[[1]]$directSeats <- directSeats
	noOfHRmComb <- 1 # number one in list input info

	# for each house size
	for (i in 1:length(housesize)) {
				
		# for each rounding method
		for (j in 1:length(appMethod)) {
		
			noOfHRmComb <- noOfHRmComb + 1
		
			# TBD call
			if (identical(listCombination, NA)) 
				tmpApp <- vectorApp(monoApp, h = housesize[i], rM = appMethod[j], pq = pq, dS = directSeats, rT = resolveTies, pN = partyNames)
			else 
				tmpApp <- listCombApp(housesize = housesize[i], 
							votes = votes, appMethod = appMethod[i], pq = pq,
							minConstr = minConstr, maxConstr = maxConstr, 
							directSeats = directSeats, listCombination = listCombination)
			
			app[[noOfHRmComb]] <- tmpApp
			namesApp <- c(namesApp, paste(appMethod[j], "h", housesize[i], sep = ""))
			
		}
	}	
	
	names(app) <- namesApp
	
	# remove reference to Java-object
	if(identical(listCombination, NA)) rm(monoApp)
	
	# return
	class(app) <- "RBaziApportionment"
	return(app)
}	


# internal function for computing an apportionment

vectorApp <- 
function(rjBaziObjRef, h, rM, pq = NULL, dS = NA, rT, pN)
# h(ousesize), r(ounding)M(ethod), d(irect)S(eats), r(esolve)T(ies), p(arty)N(ames)
{
tmpApp <- list()

	# init return object
	tmpApp$housesize <- h
	tmpApp$apportionmentMethod <- rM
	
	if (rM == "DivSta" || rM == "DivPot" || rM == "DivPow") {
		tmpApp$noOfApp <- .jcall(rjBaziObjRef, returnSig = "I", "calcApp", as.integer(h), rM, pq)
	}
	else {
		tmpApp$noOfApp <- .jcall(rjBaziObjRef, returnSig = "I", "calcApp", as.integer(h), rM)
	}	
		
	# creates return object
	if (tmpApp$noOfApp > 0) {
		tmpApp$apportionment <- .jcall(rjBaziObjRef, returnSig = "[I", "getApp")
		names(tmpApp$apportionment) <- pN
			
		tmpApp$ties <- .jcall(rjBaziObjRef, returnSig = "[I", "getFlags")
		names(tmpApp$ties) <- pN
			
		if(rM == "HaQgrR" || rM == "DrQgrR") tmpApp$quota <- .jcall(rjBaziObjRef, returnSig = "D", "getQuota")
		else tmpApp$divisor <- .jcall(rjBaziObjRef, returnSig = "[D", "getDivisor")
	
		if (!identical(dS, NA)) {
			tmpApp$overhang <- dS - tmpApp$apportionment
			tmpApp$overhang[tmpApp$overhang<0] <- 0
		}


		# get multiple apportionments
		if (rT) {
			multipleApp <- matrix(.jcall(rjBaziObjRef, returnSig = "[I", "getMultiple"), tmpApp$noOfApp, byrow = TRUE)
			rownames(multipleApp) <- mapply(paste, "A", seq(1, tmpApp$noOfApp), sep="")
			colnames(multipleApp) <- pN
			tmpApp$multipleApp <- multipleApp
		}
			
		# get error messages
		errorCode <- .jcall(rjBaziObjRef, returnSig = "I", "getErrorCode")
			
		if (rM == "DrQgrR" && all(tmpApp$apportionment == 0)) {
			tmpApp$apportionment <- rep(NA, length(tmpApp$apportionment))
			tmpApp$error <- "DrQgrR NA, cf. BAZI manual"
		}
	}
	else if (tmpApp$noOfApp == -1) {
		tmpApp$apportionment <- NA
	}
	else stop("Never come here")			
	
	return(tmpApp)
}		
