\name{bipropApp}

\alias{bipropApp}

\title{Adapter module for biproportional (matrix) apportionment using BAZI}

\description{Computes a biproportional Apportionment}

\usage{
bipropApp(districtMagnitudes, votes, appMethod = "DivStd", pq = NULL, nzz = FALSE,
          minConstr = NA, minConstrSuper = NA, algo = "ASmdpt|TTflpt",
          colM = NA, appMethodSuper = NULL, sorted = TRUE, resolveTies = FALSE, name = NULL)
}


\arguments{
	\item{districtMagnitudes}{District (row) marginals}
	\item{votes}{Matrix with input weights, e.g. votes for parties}
	\item{appMethod}{Rounding method to be used for apportionment, see details.}
	\item{pq}{\eqn{\in [0,1]}{in [0,1]} for stationary rounding \code{"DivSta"},
		\eqn{\in R}{in R} for power means rounding \code{"DivPMean"}}
	\item{nzz}{Use the Zurich version (NZZ) for superapportionment}
	\item{minConstr}{Minimum requirements, forces \code{apportionment >= minConstr}}
	\item{minConstrSuper}{Minimum requirements for superapportionment, forces
		\code{superapportionment >= max(minConstrSuper, colSums(minConstr))}}
	\item{algo}{Algorithm for computing the apportionment, see references.}
	\item{colM}{Additional party (col) marginals. Must have the same sum as
		\code{districtMagnitudes}, overrides any other option for column
		marginals like \code{minConstr} etc.}
	\item{appMethodSuper}{Rounding method for superapportionment, if different
		from rounding method subapportionment}
	\item{sorted}{Sort parties in decreasing order of number of votes}
	\item{resolveTies}{Resolve ties if there are multiple apportionments}
	\item{name}{Name for the apportionment problem}
}

\details{
The minimum setting for a biproportional apportionment is \code{districtMagnitudes,
votes} and \code{appMethod}. As algorithms for the computation of biproportional
apportionments, the following values are valid: \kbd{"ASmdpt", "ASextr", "ASrand",
"TTinte", "TTflpt","ASmdpt|TTflpt", "ASextr|TTflpt", "ASrand|TTflpt", 
"ASmdpt|TTinte", "ASextr|TTinte", "ASrand|TTinte", "IPFP|TTinte", "IPFP|TTflpt"}.
For the \kbd{appMethod} argument the different following rounding methods are valid:
rounding down (\code{"DivDwn"}) to the last integer, standard rounding 
(\code{"DivStd"}) in the "usual way", geometric rounding (\code{"DivGeo"}),
harmonic rounding (\code{"DivHar"}), rounding up (\code{"DivUp"}) to the next
integer, general stationary rounding (\code{"DivSta"}), and power means
rounding (\code{"DivPMean"}).
}

\value{
List containing either an super- and an subapportionment or a list containing all
apportionments.
}

\seealso{\code{\link{computeApportionment}}}

\examples{

# Toy example
votes <- matrix(c(12, 15,
                  17, 23,
                  33, 19), byrow=TRUE, nrow=3,
                  dimnames=list(c("D1", "D2", "D3"), c("P1", "P2")))
districtMagnitudes <- c(2, 3, 4)
bipropApp(districtMagnitudes, votes)


}

\references{
	\url{http://www.uni-augsburg.de/bazi}.
	
	Maier, S. and Pukelsheim, F. (2007) \emph{BAZI - A Free Program for Proportional
	Representation Apportionment}, Preprint No. 42, Institute for Mathematics, University of 
	Augsburg.
	
	Maier, S. Zachariassen, P. and Zachariasen, M. (2007) \emph{Divisor-Based
	Biproportional Apportionment in Electoral Systems: A Real-Life Benchmark Study},
	Preprint No. 39, Institute for Mathematics, University of Augsburg.
}	

\keyword{math}
